@@ -6,6 +6,10 @@ char * diff_with_file (const char  *file,
                        gssize       len,
                        GError     **error);
 
+gboolean file_has_prefix (const char  *file,
+                          const char  *str,
+                          GError     **error);
+
 void print_attribute (PangoAttribute *attr,
                       GString        *string);
 
@@ -15,6 +19,9 @@ void print_attributes (GSList        *attrs,
 void print_attr_list (PangoAttrList  *attrs,
                       GString        *string);
 
+PangoAttrList *
+attributes_from_string (const char *string);
+
 const char *get_script_name (GUnicodeScript s);
 
 
